"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
var GlobalEventListener = (function () {
    function GlobalEventListener() {
        var _this = this;
        this.handleEvent = function (name, event) {
            for (var id in _this.callbacks) {
                if (helpers_1.hasOwnProp(_this.callbacks, id))
                    _this.callbacks[id][name](event);
            }
        };
        this.register = function (callbacks) {
            var id = helpers_1.generateUniqueId();
            for (var i in callbacks) {
                if (helpers_1.hasOwnProp(callbacks, i)) {
                    if (helpers_1.hasOwnProp(_this.listeners, i))
                        window.removeEventListener(i, _this.listeners[id]);
                    _this.listeners[i] = _this.handleEvent.bind(_this, i);
                    window.addEventListener(i, _this.listeners[i]);
                }
            }
            _this.callbacks[id] = callbacks;
            return id;
        };
        this.unregister = function (id) {
            if (id && _this.callbacks[id]) {
                for (var i in _this.callbacks[id]) {
                    if (helpers_1.hasOwnProp(_this.callbacks[id], i) && helpers_1.hasOwnProp(_this.listeners, i)) {
                        window.removeEventListener(i, _this.listeners[i]);
                    }
                }
                delete _this.callbacks[id];
            }
        };
        this.callbacks = {};
        this.listeners = {};
    }
    GlobalEventListener.prototype.dispatchGlobalEvent = function (eventName, opts, target) {
        if (target === void 0) { target = window; }
        var event;
        if (typeof window['CustomEvent'] === 'function') {
            event = new window['CustomEvent'](eventName, { detail: opts });
        }
        else {
            event = document.createEvent('Event');
            event.initEvent(eventName, false, true, opts);
        }
        if (target) {
            target.dispatchEvent(event);
        }
    };
    return GlobalEventListener;
}());
exports.GlobalEventListener = GlobalEventListener;
exports.default = new GlobalEventListener();
//# sourceMappingURL=eventListener.js.map